@@ -7,10 +7,12 @@ import lightkurve
 import numpy
 import pandas
 import yaml
+import multiprocessing
 from scipy import stats
 from scipy.signal import savgol_filter
 from wotan import flatten
 
+from lcbuilder.helper import LcbuilderHelper
 from lcbuilder.star.starinfo import StarInfo
 
 from lcbuilder.objectinfo.InputObjectInfo import InputObjectInfo
@@ -37,7 +39,8 @@ class LcBuilder:
                                     MissionFfiIdObjectInfo: MissionFfiLightcurveBuilder(),
                                     MissionFfiCoordsObjectInfo: MissionFfiLightcurveBuilder()}
 
-    def build(self, object_info: ObjectInfo, object_dir: str, caches_root_dir=os.path.expanduser('~') + "/"):
+    def build(self, object_info: ObjectInfo, object_dir: str, caches_root_dir=os.path.expanduser('~') + "/",
+              cpus=multiprocessing.cpu_count() - 1):
         lc_build = self.lightcurve_builders[type(object_info)].build(object_info, object_dir, caches_root_dir)
         if lc_build.tpf_apertures is not None:
             with open(object_dir + "/apertures.yaml", 'w') as f:
@@ -69,20 +72,8 @@ class LcBuilder:
                                                                              flux_err_float, star_info,
                                                                              lc_build.cadence, object_dir)
         lc = lightkurve.LightCurve(time=clean_time, flux=flatten_flux, flux_err=clean_flux_err)
-        periodogram = lc.to_periodogram(minimum_period=0.05, maximum_period=15, oversample_factor=10)
-        # power_norm = self.running_median(periodogram.power.value, 20)
-        periodogram.plot(view='period', scale='log')
-        plt.title(str(sherlock_id) + " Lightcurve periodogram")
-        plt.savefig(object_dir + "/Initial_Periodogram_" + str(sherlock_id) + ".png", bbox_inches='tight')
-        plt.clf()
-        plt.close()
-        # power_mod = periodogram.power.value - power_norm
-        # power_mod = power_mod / np.mean(power_mod)
-        # periodogram.power = power_mod * u.d / u.d
-        # periodogram.plot(view='period', scale='log')
-        # plt.title(str(sherlock_id) + " Lightcurve normalized periodogram")
-        # plt.savefig(object_dir + "PeriodogramNorm_" + str(sherlock_id) + ".png", bbox_inches='tight')
-        # plt.clf()
+        periodogram = self.__plot_periodogram(lc, 0.05, 15, 10, sherlock_id, 
+                                              object_dir + "/Periodogram_Initial_" + str(sherlock_id) + ".png")
         if object_info.auto_detrend_period is not None:
             lc_build.detrend_period = object_info.auto_detrend_period
         elif object_info.auto_detrend_enabled:
@@ -92,10 +83,12 @@ class LcBuilder:
             logging.info('STELLAR OSCILLATIONS REDUCTION')
             logging.info('================================================')
             flatten_flux = self.__reduce_simple_oscillations(object_dir, object_info.mission_id(), clean_time,
-                                                             flatten_flux, object_info.oscillation_snr_threshold,
+                                                             flatten_flux, star_info,
+                                                             object_info.oscillation_snr_threshold,
                                                              object_info.oscillation_amplitude_threshold,
                                                              object_info.oscillation_ws_scale,
-                                                             object_info.oscillation_min_period)
+                                                             object_info.oscillation_min_period,
+                                                             cpus)
         if lc_build.detrend_period is not None:
             logging.info('================================================')
             logging.info('AUTO-DETREND EXECUTION')
@@ -136,8 +129,28 @@ class LcBuilder:
                 clean_flux_err = clean_flux_err[~mask]
         lc = lightkurve.LightCurve(time=clean_time, flux=flatten_flux, flux_err=clean_flux_err)
         lc_build.lc = lc
+        self.__plot_periodogram(lc, 0.05, 15, 10, sherlock_id,
+                                object_dir + "/Periodogram_Final_" + str(sherlock_id) + ".png")
         return lc_build
 
+    def __plot_periodogram(self, lc, min_period, max_period, oversample, object_id, filename):
+        periodogram = lc.to_periodogram(minimum_period=min_period, maximum_period=max_period,
+                                        oversample_factor=oversample)
+        # power_norm = self.running_median(periodogram.power.value, 20)
+        periodogram.plot(view='period', scale='log')
+        plt.title(str(object_id) + " Lightcurve periodogram")
+        plt.savefig(filename, bbox_inches='tight')
+        plt.clf()
+        plt.close()
+        # power_mod = periodogram.power.value - power_norm
+        # power_mod = power_mod / np.mean(power_mod)
+        # periodogram.power = power_mod * u.d / u.d
+        # periodogram.plot(view='period', scale='log')
+        # plt.title(str(sherlock_id) + " Lightcurve normalized periodogram")
+        # plt.savefig(object_dir + "PeriodogramNorm_" + str(sherlock_id) + ".png", bbox_inches='tight')
+        # plt.clf()
+        return periodogram
+
     def smooth(self, flux, window_len=11, window='blackman'):
         clean_flux = savgol_filter(flux, window_len, 3)
         # if window_len < 3:
@@ -168,14 +181,17 @@ class LcBuilder:
             period = None
         return period
 
-    def __reduce_simple_oscillations(self, object_dir, object_id, time, flux, snr_threshold=4, amplitude_threshold=0.1,
-                                     window_size_scale=60, oscillation_min_period=0.001):
+    def __reduce_simple_oscillations(self, object_dir, object_id, time, flux, star_info, snr_threshold=4,
+                                     amplitude_threshold=0.1, window_size_scale=60, oscillation_min_period=0.002,
+                                     oscillation_max_period=0.2, cpus=multiprocessing.cpu_count() - 1):
+        no_transits_time, no_transits_flux = self.__reduce_visible_transits(time, flux, star_info, cpus)
         snr = 10
         number = 0
         pulsations_df = pandas.DataFrame(columns=['period_s', 'frequency_microHz', 'amplitude', 'phase', 'snr',
                                                   'number'])
-        lc = lightkurve.LightCurve(time=time, flux=flux)
-        periodogram = lc.to_periodogram(minimum_period=oscillation_min_period, maximum_period=2, oversample_factor=1)
+        lc = lightkurve.LightCurve(time=no_transits_time, flux=no_transits_flux)
+        periodogram = lc.to_periodogram(minimum_period=oscillation_min_period, maximum_period=oscillation_max_period,
+                                        oversample_factor=1)
         remove_signal = snr > snr_threshold
         sa_dir = object_dir + "sa/"
         while remove_signal:
@@ -192,33 +208,36 @@ class LcBuilder:
             if remove_signal:
                 if not os.path.exists(sa_dir):
                     os.mkdir(sa_dir)
-                guess_amp = numpy.std(flux) * 2. ** 0.5
+                guess_amp = numpy.std(no_transits_flux) * 2. ** 0.5
                 guess = numpy.array([guess_amp, 0.])
 
                 def sinfunc(t, a, p):
                     return a * numpy.sin(omega * t + p) + 1
 
-                popt, pcov = scipy.optimize.curve_fit(sinfunc, time, flux, p0=guess)
+                popt, pcov = scipy.optimize.curve_fit(sinfunc, no_transits_time, no_transits_flux, p0=guess)
                 perr = numpy.sqrt(numpy.diag(pcov))
                 A_err, p_err = perr
                 A, p = popt
                 fitfunc = lambda t: A * numpy.sin(omega * t + p) + 1
                 fit_flux = fitfunc(time)
+                fit_no_transit_flux = fitfunc(no_transits_time)
                 flux_corr = flux - fit_flux + 1
                 A = numpy.sqrt(A ** 2)
                 remove_signal = self.__is_simple_oscillation_good_enough(snr, snr_threshold, A, A_err,
                                                                          p, p_err, numpy.std(flux),
                                                                          numpy.std(flux_corr), amplitude_threshold)
                 if remove_signal:
-                    logging.info("Reducing pulsation with period %sd, flux amplitude of %s, phase minima at %s and snr %s",
-                                 period, A, p, snr)
+                    logging.info(
+                        "Reducing pulsation with period %sd, flux amplitude of %s, phase at %s and snr %s",
+                        period, A, p, snr)
                     pulsations_df = pulsations_df.append(
                         {'period_s': period * 24 * 3600, 'frequency_microHz': frequency / 24 / 3600 * 1000000,
                          'amplitude': A, 'phase': p, 'snr': snr, 'number': number},
                         ignore_index=True)
                     self.__plot_pulsation_fit(sa_dir, object_id, time, flux, fit_flux, period, number)
                     flux = flux_corr
-                    lc = lightkurve.LightCurve(time=time, flux=flux)
+                    no_transits_flux = no_transits_flux - fit_no_transit_flux + 1
+                    lc = lightkurve.LightCurve(time=no_transits_time, flux=no_transits_flux)
                     periodogram = lc.to_periodogram(minimum_period=oscillation_min_period, maximum_period=2,
                                                     oversample_factor=1)
                     self.__plot_pulsation_periodogram(sa_dir, object_id, periodogram, period, number)
@@ -228,6 +247,40 @@ class LcBuilder:
             pulsations_df.to_csv(sa_dir + "signals.csv", index=False)
         return flux
 
+    def __reduce_visible_transits(self, time, flux, star_info, cpus):
+        min_sde = 13
+        sde = min_sde + 1
+        logging.info("Searching for obvious transits to mask them before reducing the stellar pulsations")
+        while sde > min_sde:
+            model = tls.transitleastsquares(time, flux)
+            transit_period_min = 0.3
+            transit_period_max = 2
+            power_args = {"period_min": 0.3,
+                          "period_max": 2, "n_transits_min": 2, "show_progress_bar": False,
+                          "duration_grid_step": 1.15,
+                          "use_threads": cpus, "oversampling_factor": 3,
+                          "period_grid": LcbuilderHelper.calculate_period_grid(time, transit_period_min,
+                                                                               transit_period_max, 1, star_info, 2)}
+            if star_info.ld_coefficients is not None:
+                power_args["u"] = star_info.ld_coefficients
+            power_args["R_star"] = star_info.radius
+            power_args["R_star_min"] = star_info.radius_min
+            power_args["R_star_max"] = star_info.radius_max
+            power_args["M_star"] = star_info.mass
+            power_args["M_star_min"] = star_info.mass_min
+            power_args["M_star_max"] = star_info.mass_max
+            results = model.power(**power_args)
+            sde = results.SDE
+            if sde > min_sde:
+                logging.info("Masking transit at period %.2f d, T0 %.2f and duration %.2f m.", results.period,
+                             results.T0,
+                             results.duration * 60 * 24)
+                in_transit = tls.transit_mask(time, results.period,
+                                              results.duration if results.duration > 0.01 else 0.01, results.T0)
+                time = time[~in_transit]
+                flux = flux[~in_transit]
+        return time, flux
+
     def __is_simple_oscillation_good_enough(self, snr, snr_threshold, A, A_err, p, p_err, flux_std, flux_corr_std,
                                             amplitude_threshold):
         return snr > snr_threshold and (A_err == numpy.inf or (A_err / A < 0.2)) and \
@@ -394,15 +447,15 @@ class LcBuilder:
                 time_partial = clean_time[previous_jump_index:jumpIndex]
                 flux_partial = clean_flux[previous_jump_index:jumpIndex]
                 before_flux_partial = before_flux[previous_jump_index:jumpIndex]
-                bins = (time_partial[len(time_partial) - 1] - time_partial[0]) * bins_per_day
-                bin_stds, bin_edges, binnumber = stats.binned_statistic(time_partial, flux_partial, statistic='std',
+                bins = (time_partial[len(time_partial) - 1] - time_partial[1]) * bins_per_day
+                bin_stds, bin_edges, binnumber = stats.binned_statistic(time_partial[1:], flux_partial[1:], statistic='std',
                                                                         bins=bins)
                 stds_median = numpy.nanmedian(bin_stds[bin_stds > 0])
                 stds_median_array = numpy.full(len(bin_stds), stds_median)
                 rms_threshold_array = stds_median_array * object_info.high_rms_threshold
                 too_high_bin_stds_indexes = numpy.argwhere(bin_stds > rms_threshold_array)
                 high_std_mask = numpy.array([bin_id - 1 in too_high_bin_stds_indexes for bin_id in binnumber])
-                entire_high_rms_mask = numpy.append(entire_high_rms_mask, high_std_mask)
+                entire_high_rms_mask = numpy.append(entire_high_rms_mask, numpy.append(high_std_mask[0], high_std_mask))
                 bin_width = (bin_edges[1] - bin_edges[0])
                 bin_centers = bin_edges[1:] - bin_width / 2
                 entire_bin_centers = numpy.append(entire_bin_centers, bin_centers)
@@ -414,8 +467,11 @@ class LcBuilder:
                                      'High_RMS_Mask_' + str(star_info.object_id) + '_time_' +
                                      str(time_partial[1]) + '_' + str(time_partial[-1]), object_dir)
             self.__plot_rms_mask(star_info.object_id, object_info.high_rms_bin_hours, entire_bin_centers,
-                                 entire_bin_stds, entire_rms_threshold_array, entire_high_rms_mask, clean_time,
+                                 entire_bin_stds, entire_rms_threshold_array, entire_high_rms_mask[1:], clean_time,
                                  clean_flux, 'High_RMS_Mask_' + str(star_info.object_id), object_dir)
+            clean_time = clean_time[~entire_high_rms_mask]
+            clean_flux = clean_flux[~entire_high_rms_mask]
+            clean_flux_err = clean_flux_err[~entire_high_rms_mask]
         if is_short_cadence and object_info.smooth_enabled:
             # logging.info('Applying Smooth phase (savgol + weighted average)')
             logging.info('Applying Smooth phase (savgol)')
@@ -446,7 +502,7 @@ class LcBuilder:
         axs[0].plot(bin_centers, rms_threshold_array, color='red', rasterized=True,
                     label='Mask Threshold')
         axs[1].scatter(time[1:], flux[1:], color='gray', alpha=0.5, rasterized=True, label="Flux norm.")
-        axs[1].scatter(time[rms_mask][1:], flux[rms_mask][1:], linewidth=1, color='red',
+        axs[1].scatter(time[1:][rms_mask], flux[1:][rms_mask], linewidth=1, color='red',
                        alpha=1.0,
                        label="High RMS")
         axs[0].legend(loc="upper right")
